from __future__ import annotations
from typing import TYPE_CHECKING

from pipen import plugin
from pipen.utils import get_logger
from filelock import FileLock, Timeout

if TYPE_CHECKING:  # pragma: no cover
    from pipen import Proc

__version__ = "0.7.0"

logger = get_logger("lock", "info")


class PipenLockPlugin:
    version = __version__
    name = "lock"

    def __init__(self):
        self.lock = None

    @plugin.impl
    async def on_proc_start(self, proc: Proc):
        self.lock = FileLock(proc.workdir / "proc.lock")
        try:
            self.lock.acquire(blocking=False)
        except Timeout:
            proc.log(
                "warning",
                "Process locked, likely handled by another pipeline instance",
                logger=logger,
            )
        else:
            self.lock.release()

        self.lock.acquire()

    @plugin.impl
    async def on_proc_done(self, proc: Proc, succeeded: bool):
        self.lock.release(True)


pipen_lock_plugin = PipenLockPlugin()



# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['pipen_lock']
install_requires = \
['filelock>=3,<4', 'pipen>=0.14,<0.15']

entry_points = \
{'pipen': ['lock = pipen_lock:pipen_lock_plugin']}

setup_kwargs = {
    'name': 'pipen-lock',
    'version': '0.7.0',
    'description': 'Process lock for pipen to prevent multiple runs at the same time',
    'long_description': '# pipen-lock\n\nProcess lock for pipen to prevent multiple runs at the same time\n\n## Installation\n\n```bash\npip install -U pipen-lock\n```\n\n## Enable/Disable\n\nThe plugin is enabled by default. To disable it, either uninstall it or:\n\n```python\nfrom pipen import Proc, Pipen\n\n# process definition\n\nclass MyPipeline(Pipen):\n    plugins = ["no:lock"]\n\n```\n',
    'author': 'pwwang',
    'author_email': 'pwwang@pwwang.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
